import esbuild from "esbuild";
import slash from "slash";
import { fileURLToPath } from "url";
import { cachedCompilation, getCachedSource } from "../core/compile/index.js";
import {
  isRelativePath,
  prependForwardSlash,
  removeLeadingForwardSlashWindows,
  startsWithForwardSlash
} from "../core/path.js";
import { viteID } from "../core/util.js";
import { getFileInfo, normalizeFilename } from "../vite-plugin-utils/index.js";
import { handleHotUpdate } from "./hmr.js";
import { parseAstroRequest } from "./query.js";
const FRONTMATTER_PARSE_REGEXP = /^\-\-\-(.*)^\-\-\-/ms;
function astro({ settings, logging }) {
  const { config } = settings;
  let resolvedConfig;
  const srcRootWeb = config.srcDir.pathname.slice(config.root.pathname.length - 1);
  const isBrowserPath = (path) => path.startsWith(srcRootWeb) && srcRootWeb !== "/";
  const isFullFilePath = (path) => path.startsWith(prependForwardSlash(slash(fileURLToPath(config.root))));
  function relativeToRoot(pathname) {
    const arg = startsWithForwardSlash(pathname) ? "." + pathname : pathname;
    const url = new URL(arg, config.root);
    return slash(fileURLToPath(url)) + url.search;
  }
  function resolveRelativeFromAstroParent(id, parsedFrom) {
    const filename = normalizeFilename(parsedFrom.filename, config);
    const resolvedURL = new URL(id, `file://${filename}`);
    const resolved = resolvedURL.pathname;
    if (isBrowserPath(resolved)) {
      return relativeToRoot(resolved + resolvedURL.search);
    }
    return slash(fileURLToPath(resolvedURL)) + resolvedURL.search;
  }
  return {
    name: "astro:build",
    enforce: "pre",
    configResolved(_resolvedConfig) {
      resolvedConfig = _resolvedConfig;
    },
    async resolveId(id, from, opts) {
      if (from) {
        const parsedFrom = parseAstroRequest(from);
        const isAstroScript = parsedFrom.query.astro && parsedFrom.query.type === "script";
        if (isAstroScript && isRelativePath(id)) {
          return this.resolve(resolveRelativeFromAstroParent(id, parsedFrom), from, {
            custom: opts.custom,
            skipSelf: true
          });
        }
      }
      const { query } = parseAstroRequest(id);
      if (query.astro) {
        if (query.type === "style" && isBrowserPath(id)) {
          return relativeToRoot(id);
        }
        if (query.type === "style" && id.startsWith("/@fs")) {
          id = removeLeadingForwardSlashWindows(id.slice(4));
        }
        if (isFullFilePath(id)) {
          return viteID(new URL("file://" + id));
        }
        return id;
      }
    },
    async load(id, opts) {
      const parsedId = parseAstroRequest(id);
      const query = parsedId.query;
      if (!query.astro) {
        return null;
      }
      let filename = parsedId.filename;
      let raw = await this.resolve(filename, void 0);
      if (!raw) {
        return null;
      }
      let source = getCachedSource(config, raw.id);
      if (!source) {
        return null;
      }
      const compileProps = {
        astroConfig: config,
        viteConfig: resolvedConfig,
        filename,
        source
      };
      switch (query.type) {
        case "style": {
          if (typeof query.index === "undefined") {
            throw new Error(`Requests for Astro CSS must include an index.`);
          }
          const transformResult = await cachedCompilation(compileProps);
          const csses = transformResult.css;
          const code = csses[query.index];
          return {
            code,
            meta: {
              vite: {
                isSelfAccepting: true
              }
            }
          };
        }
        case "script": {
          if (typeof query.index === "undefined") {
            throw new Error(`Requests for hoisted scripts must include an index`);
          }
          if (opts == null ? void 0 : opts.ssr) {
            return {
              code: `/* client hoisted script, empty in SSR: ${id} */`
            };
          }
          const transformResult = await cachedCompilation(compileProps);
          const scripts = transformResult.scripts;
          const hoistedScript = scripts[query.index];
          if (!hoistedScript) {
            throw new Error(`No hoisted script at index ${query.index}`);
          }
          if (hoistedScript.type === "external") {
            const src = hoistedScript.src;
            if (src.startsWith("/") && !isBrowserPath(src)) {
              const publicDir = config.publicDir.pathname.replace(/\/$/, "").split("/").pop() + "/";
              throw new Error(
                `

<script src="${src}"> references an asset in the "${publicDir}" directory. Please add the "is:inline" directive to keep this asset from being bundled.

File: ${filename}`
              );
            }
          }
          let result = {
            code: "",
            meta: {
              vite: {
                lang: "ts"
              }
            }
          };
          switch (hoistedScript.type) {
            case "inline": {
              let { code, map } = hoistedScript;
              result.code = appendSourceMap(code, map);
              break;
            }
            case "external": {
              const { src } = hoistedScript;
              result.code = `import "${src}"`;
              break;
            }
          }
          return result;
        }
        default:
          return null;
      }
    },
    async transform(source, id) {
      var _a;
      const parsedId = parseAstroRequest(id);
      if (!id.endsWith(".astro") || parsedId.query.astro) {
        return;
      }
      if (isRelativePath(parsedId.filename)) {
        return;
      }
      const filename = normalizeFilename(parsedId.filename, config);
      const compileProps = {
        astroConfig: config,
        viteConfig: resolvedConfig,
        filename,
        source
      };
      try {
        const transformResult = await cachedCompilation(compileProps);
        const { fileId: file, fileUrl: url } = getFileInfo(id, config);
        for (const dep of transformResult.cssDeps) {
          this.addWatchFile(dep);
        }
        const { code, map } = await esbuild.transform(transformResult.code, {
          loader: "ts",
          sourcemap: "external",
          sourcefile: id,
          define: (_a = config.vite) == null ? void 0 : _a.define
        });
        let SUFFIX = "";
        SUFFIX += `
const $$file = ${JSON.stringify(file)};
const $$url = ${JSON.stringify(
          url
        )};export { $$file as file, $$url as url };
`;
        if (!resolvedConfig.isProduction) {
          let i = 0;
          while (i < transformResult.scripts.length) {
            SUFFIX += `import "${id}?astro&type=script&index=${i}&lang.ts";`;
            i++;
          }
        }
        if (!resolvedConfig.isProduction) {
          SUFFIX += `
if (import.meta.hot) { import.meta.hot.decline() }`;
        }
        const astroMetadata = {
          clientOnlyComponents: transformResult.clientOnlyComponents,
          hydratedComponents: transformResult.hydratedComponents,
          scripts: transformResult.scripts
        };
        return {
          code: `${code}${SUFFIX}`,
          map,
          meta: {
            astro: astroMetadata,
            vite: {
              lang: "ts"
            }
          }
        };
      } catch (err) {
        const scannedFrontmatter = FRONTMATTER_PARSE_REGEXP.exec(source);
        if (scannedFrontmatter) {
          try {
            await esbuild.transform(scannedFrontmatter[1], {
              loader: "ts",
              sourcemap: false,
              sourcefile: id
            });
          } catch (frontmatterErr) {
            if (frontmatterErr && frontmatterErr.message) {
              frontmatterErr.message = frontmatterErr.message.replace(
                "end of file",
                "end of frontmatter"
              );
            }
            throw frontmatterErr;
          }
        }
        if (err.stack && err.stack.includes("wasm-function")) {
          const search = new URLSearchParams({
            labels: "compiler",
            title: "\u{1F41B} BUG: `@astrojs/compiler` panic",
            template: "---01-bug-report.yml",
            "bug-description": `\`@astrojs/compiler\` encountered an unrecoverable error when compiling the following file.

**${id.replace(fileURLToPath(config.root), "")}**
\`\`\`astro
${source}
\`\`\``
          });
          err.url = `https://github.com/withastro/astro/issues/new?${search.toString()}`;
          err.message = `Error: Uh oh, the Astro compiler encountered an unrecoverable error!

    Please open
    a GitHub issue using the link below:
    ${err.url}`;
          if (logging.level !== "debug") {
            err.stack = `    at ${id}`;
          }
        }
        throw err;
      }
    },
    async handleHotUpdate(context) {
      if (context.server.config.isProduction)
        return;
      const compileProps = {
        astroConfig: config,
        viteConfig: resolvedConfig,
        filename: context.file,
        source: await context.read()
      };
      const compile = () => cachedCompilation(compileProps);
      return handleHotUpdate(context, {
        config,
        logging,
        compile
      });
    }
  };
}
function appendSourceMap(content, map) {
  if (!map)
    return content;
  return `${content}
//# sourceMappingURL=data:application/json;charset=utf-8;base64,${Buffer.from(
    map
  ).toString("base64")}`;
}
export {
  astro as default
};
